<?php

namespace Ecommerce\Util;

use Exception\CustomException;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\Crypt;

class LoginUtilities 
{
    const LOGIN_CRYPT_STRING = '1460_loginManagement';
    
    public static function getLoginConfig() {
        return json_decode(file_get_contents(__DIR__ . '/../../../config/config.json'), true);
    }
    
    public static function getLoginIntegration() {
        return json_decode(file_get_contents(__DIR__ . '/../../../config/integration.json'), true)[0];
    }
    
    /**
     * @param $token
     * @param $requestType
     * @param $data
     * @param $dataSetResultPos
     * @param $privateKey
     * @param $url
     * @param bool $keepConnected
     * @param bool $isThrowNotConnectedErrors
     * @return array
     * @throws \Exception
     */
    public function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected = false, $isThrowNotConnectedErrors = false)
    {
        set_time_limit(180);
        $notConnectCUrlErrorCodes = array(
            7,  //Failed to connect to host or proxy
            28, //Operation timeout
            35, //SSL connection error
            89, //No connection available
        );
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $privateKey;
        if($keepConnected) {
            $header[] = 'OAuth-KeepConnected: ' . $keepConnected;
        }
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $requestType);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);

        $resultCurl = curl_exec($ch);
        $result     = json_decode($resultCurl, true);
        $error      = curl_error($ch);
        $errorCode  = curl_errno($ch);
        curl_close($ch);
        
        if (!empty($result['error'])) {
            $errorCode = isset($result['errorCode']) ? $result['errorCode'] : $errorCode;
            throw new \Exception($result['error'], $errorCode);
        } else if (!empty($error)) {
            if(in_array($errorCode, $notConnectCUrlErrorCodes) && !$isThrowNotConnectedErrors) {
                return null;
            }
            throw new \Exception($error, $errorCode);
        } else if(is_null($result) && $isThrowNotConnectedErrors) {
            throw new \Exception($resultCurl, $errorCode);
        } else {
            return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
        }
    }
    
    public function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            
            if ($name == $nameFilter ) {
                if ($remove) {
                    unset($filters[$key]);
                }
                
                return $values;
            }
        }
        
        return null;
    }
    
    public function generateRandomToken($length)
    {
        $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $charactersLength = strlen($characters);
        $randomPassword = '';
        
        for ($i = 1; $i <= $length; $i++) {
            $rand = mt_rand(1, $charactersLength);
            $randomPassword .= $characters[$rand-1];
        }
        
        return $randomPassword;
    }
    
    public function zeedhiEncrypt($text, $key = 'login') {
        return Crypt::encrypt($text, $key );
    }

    public function zeedhiDecrypt($text, $key = 'login') {
        return Crypt::decrypt($text, $key );
    }
    
    /**
     * Capitalize the words of the string sent.
     * 
     * @param string $word String that will be capitalized.  
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function capitalizeWords($word)
    {
        return ucwords(strtolower($word));
    }
    
    /**
     * @param CustomException $e
     * @return Error
     */
    public function getErrorObjectFromException(CustomException $e) 
    {
        $error = array (
            'CODE'   => $e->getMessage(),
            'PARAMS' => $e->getParams()
        );
        
        return new Error(json_encode($error), $e->getCode(), $e->getTraceAsString());
    }
}